//
// Combustion_Library.h
// This header file contains all functions.

// It is possible to break this
// file into several header files - associated with each cpp code.   
// 
// Possible header files are indicated below.  For example Reynolds_Calls.h
// can be a header file for Reyonlds_Calls.cpp

// Reynolds_Calls.h



// This Win32 dynamic link library was built with __stdcall and is
// intended for use in any C++, Visual Basic, or Fortran executable.
// You may add this header
// as is to your project if it is built with __stdcall.  To use with 
// __cdecl, you must add the __stdcall qualifier in front of the 
// functions. 
//
// Implementation of thermo properties described in "Thermodynamic
// Properties in SI," W.C. Reynolds, 1979, Stanford University.
// These properties have English units, with mass quantities in 
// lb-moles instead of kilograms.  Arguments are also English.  The 
// reference state is the pure ideal gas in the standard state (1 atm)
// at 77F (298.15K).
//




// ============= Component Enthalpy/Entropy Functions ==================
// Enthalpy and entropy for the individual products of a hydrocarbon
// combustion reaction.  Molar properties (lb-moles).
// =====================================================================
//		Inputs
//			P		Pressure (psia)
//			T		Temperature (R)
//		Outputs
//			H_		Enthalpy of the pure component (Btu/lbmole)
//			S_		Entropy of the pure component (Btu/lbmole-R)
// =====================================================================
double H_Air( double P, double T );
double H_Argon( double P, double T );
double H_Nitrogen( double P, double T );
double H_CarbDiox( double P, double T );
double H_Water( double P, double T );
double H_Steam( double P, double T );

double H_Ammonia( double P, double T );
double H_Butane( double P, double T );
double H_Ethane( double P, double T );
double H_Ethylene( double P, double T );
double H_Hydrogen( double P, double T );
double H_Methane( double P, double T );
double H_Octane( double P, double T );
double H_Oxygen( double P, double T );
double H_Propane( double P, double T );



// =====================================================================
double S_Air( double P, double T );
double S_Argon( double P, double T );
double S_Nitrogen( double P, double T );
double S_CarbDiox( double P, double T );
double S_Water( double P, double T );
double S_Steam( double P, double T );

double S_Ammonia( double P, double T );
double S_Butane( double P, double T );
double S_Ethane( double P, double T );
double S_Ethylene( double P, double T );
double S_Hydrogen( double P, double T );
double S_Methane( double P, double T );
double S_Octane( double P, double T );
double S_Oxygen( double P, double T );
double S_Propane( double P, double T );



// =====================================================================
// ============= Pure Component Permutation Functions ==================
// Temperature or Pressure calculation functions for given Enthalpy
// or Entropy with Pressure or Temperature
// =====================================================================
//		Inputs
//			P,(T)		Pressure (psia) (Temperature (R))
//			H			Enthalpy of the pure component (Btu/lbmole)
//			S			Entropy of the pure component (Btu/lbmole-R)
//		Outputs
//			T, (P)		Temperature (R) (Pressure (psia))
// =====================================================================
double T_HP_Air( double H, double P ); 
double T_HP_Nitrogen( double H, double P ); 
double T_HP_Argon( double H, double P ); 
double T_HP_CarbDiox( double H, double P );
double T_HP_Ammonia( double H, double P );
double T_HP_Butane( double H, double P );
double T_HP_Ethane( double H, double P );
double T_HP_Ethylene( double H, double P );
double T_HP_Hydrogen( double H, double P );
double T_HP_Methane( double H, double P );
double T_HP_Octane( double H, double P );
double T_HP_Oxygen( double H, double P );
double T_HP_Propane( double H, double P );
double T_HP_Water( double H, double P);
double T_HP_Steam( double H, double P );


double T_SP_Air( double S, double P ); 
double T_SP_Nitrogen( double S, double P ); 
double T_SP_Argon( double S, double P ); 
double T_SP_CarbDiox( double S, double P );
double T_SP_Ammonia( double S, double P );
double T_SP_Butane( double S, double P );
double T_SP_Ethane( double S, double P );
double T_SP_Ethylene( double S, double P );
double T_SP_Hydrogen( double S, double P );
double T_SP_Methane( double S, double P );
double T_SP_Octane( double S, double P );
double T_SP_Oxygen( double S, double P );
double T_SP_Propane( double S, double P );
double T_SP_Water( double S, double P);
double T_SP_Steam( double S, double P );


double P_HT_Air( double H, double T ); 
double P_HT_Nitrogen( double H, double T ); 
double P_HT_Argon( double H, double T );
double P_HT_CarbDiox( double H, double T );
double P_HT_Ammonia( double H, double T );
double P_HT_Butane( double H, double T );
double P_HT_Ethane( double H, double T );
double P_HT_Ethylene( double H, double T );
double P_HT_Hydrogen( double H, double T );
double P_HT_Methane( double H, double T );
double P_HT_Octane( double H, double T );
double P_HT_Oxygen( double H, double T );
double P_HT_Propane( double H, double T );
double P_HT_Water( double H, double T );
double P_HT_Steam( double H, double T );

double P_ST_Air( double S, double T ); 
double P_ST_Nitrogen( double S, double T ); 
double P_ST_Argon( double S, double T );
double P_ST_CarbDiox( double S, double T );
double P_ST_Ammonia( double S, double T );
double P_ST_Butane( double S, double T );
double P_ST_Ethane( double S, double T );
double P_ST_Ethylene( double S, double T );
double P_ST_Hydrogen( double S, double T );
double P_ST_Methane( double S, double T );
double P_ST_Octane( double S, double T );
double P_ST_Oxygen( double S, double T );
double P_ST_Propane( double S, double T );
double P_ST_Water( double S, double T );
double P_ST_Steam( double S, double T );





// =====================================================================
// The following three functions return saturation temperature (R),
// pressure (psia) and saturation density (ft3/lbmole) respectively
// =====================================================================
double Tsat_Water( double P );
double Psat_Water( double T );
double rhosat_Water( double T );
// =====================================================================

double P_Air( double V, double T );
double P_Argon( double V, double T );
double P_Nitrogen( double V, double T );
double P_CarbDiox( double V, double T );
double P_Water( double V, double T );
double P_Ammonia( double V, double T );
double P_Butane( double V, double T );
double P_Ethane( double V, double T );
double P_Ethylene( double V, double T );
double P_Hydrogen( double V, double T );
double P_Methane( double V, double T );
double P_Octane( double V, double T );
double P_Oxygen( double V, double T );
double P_Propane( double V, double T );



//

double V_EOS( double P, double T, double (*PFunc)( double, double ), double InitGuess );

double V_Air( double P, double T );
double V_Argon( double P, double T );
double V_Nitrogen( double P, double T );
double V_CarbDiox( double P, double T );
double V_Steam( double P, double T );
double V_Water( double P, double T );

double V_Ammonia( double P, double T );
double V_Butane( double P, double T );
double V_Ethane( double P, double T );
double V_Ethylene( double P, double T );
double V_Hydrogen( double P, double T );
double V_Methane( double P, double T );
double V_Octane( double P, double T );
double V_Oxygen( double P, double T );
double V_Propane( double P, double T );

double T_XP( double X, double P, double (*XFunc)( double, double ), double InitGuess );
double P_XT( double X, double T, double (*XFunc)( double, double ), double InitGuess );



// Reynolds_Skeletons.h



//
// This Win32 static library was built with __stdcall and is
// intended for use in other C++ executables. 
//  You may add this header as is to your project
// if it is built with __stdcall.  To use with __cdecl, you must 
// add the __stdcall qualifier in front of the functions.
//
// Implementation of thermo functions described in "Thermodynamic
// Properties in SI," W.C. Reynolds, 1979, Stanford University.
// These functions were converted to English units, with mass 
// quantities in lb-moles instead of kilograms.  The reference 
// state is the pure ideal gas in the standard state (1 atm) at 
// 77F (298.15K).
//
// These files were created for use in turbine and compressor
// calculations and so include only the functions that are associated
// with air and combustion products: nitrogen, oxygen (Cv only), argon, 
// and carbon dioxide.  Water properties can be imported.



// ===================== EOS Pressure Functions ========================
// Equations of state giving pressure from volume, temperature, and 
// component-specific constants (A and Mw).  See Reynolds for specifics.
// Gamma is included in the array as the 0th element ( A[0] ).
// =====================================================================
//		Inputs
//			V,T		Molar volume and temperature (ft3/lbmol, R).
//			A[]		EOS constants including gamma as A[0].
//			Mw		Molecular weight.
//		Outputs
//			P3		Pressure calculated from equation P-3 (psia).
//			P4		Pressure calculated from equation P-4.
// =====================================================================
double PressureEOS_P2( double V, double T, const double A[], double Mw );
double PressureEOS_P3( double V, double T, const double A[], double Mw );
double PressureEOS_P4( double V, double T, const double A[], double Mw );
double PressureEOS_P6( double V, double T, const double AA[][8],  
					   const double rho_a[], double Tcrit, double Mw );
double PressureEOS_P6Q1( double V, double T, const double AA[][7],
						 double Tcrit, double Mw );

// =====================================================================

// ===================== Saturated Property Functions ==================
// Equations to calculate saturation pressure and saturated liquid density.
// See Reynolds for specifics.a and Tp are included in the array as 
// the 0th and 9th element, respectively ( a=F[0], Tp=F[9] ).
// =====================================================================
//		Inputs
//			T		Temperature (R).
//			F[]		Saturation pressure constants.
//			D[]		Saturated liquid density constants.
//			Tcrit	Critical temperature (K).
//			Pcrit	Critical pressure (MPa).
//			rhocrit Critical density (kg/m3).
//			Mw		Molecular weight.
//		Outputs
//			S6		Saturation Pressure calculated from equation S-6 (psia).
//			D5		Saturated Liquid Density calculated from equation D-5 (lbmole/ft3).
// =====================================================================
double SatPressure_S6( double T, const double F[], double Tcrit, double Pcrit );
double SLDensity_D5( double T, const double D[], double Tcrit, double rhocrit, double Mw );

// =====================================================================
// =================== Saturation Properties for Pure Components ========
// These return pure component saturation pressure and satureted liquid 
//density at the given temperature.  
// ======================================================================
//		Inputs
//			in addition to the above explained inputs
//			SatP_Eqn	equation switch (saturation pressure)
//			SatLrho_Eqn equation switch (saturated liquid density)
//		Outputs
//			Pressure	Pure component saturation pressure (psia).
//			Density		Pure component sat. liquid density  (lbmol/ft3).
// ======================================================================
double PureSatPressure( double T, double Mw, const double F[], 
					    double Tcrit, double Pcrit, int SatP_Eqn);
double PureSatLDensity( double T, double Mw, const double D[], 
					    double Tcrit, double rhocrit, int SatLrho_Eqn);

// ======================================================================

// =================== Pure Entropy and Enthalpy ========================
// These return pure component (NOT air) enthalpy or entropy at the given
// molar volume and temperature.  They are based on an ideal contribution
// from Cv plus a residual contribution calculated from the EOS.  See 
// Reynolds for an explanation of the integration path.
// ======================================================================
//		Inputs
//			V,T		Molar volume and temperature (ft3/lbmol, R).
//			Mw		Molecular weight.
//			A[]		EOS constants including gamma as A[0].
//			G[]		Cv constants including beta as G[0] for C-4 or G1 as
//						G[0] if Cv is a constant for C-0. 
//			Href,	Reference value used to set H and S to zero at any
//			Sref		user defined reference state (T,V).
//			EOS_,	Integer representing the EOS and Cv equations to be
//			Cv_Eqn		used in the calculations, i.e. C-4 is Cv_Eqn = 4.
//		Outputs
//			Enth.	Pure component enthalpy (Btu/lbmol).
//			Entr.	Pure component entropy  (Btu/lbmol-R).
// ======================================================================
double PureEnthalpy( double V, double T, double Mw, const double A[], 
					 const double G[], double Href, int EOS_Eqn, 
					 int Cv_Eqn);
double PureEnthalpyP6( double V, double T, double Mw, 
					 const double AA[][8], const double rho_a[], 
					 const double G[], double Href, double Tcrit, 
					 int Cv_Eqn);
double PureEnthalpyP6Q1( double V, double T, double Mw, 
					 const double AA[][7], const double G[], 
					 double Href, double Tcrit, int Cv_Eqn  );
double PureEnthalpyPoly( double T, double Mw, 
					 const double A[], double Href);

double PureEntropy( double V, double T, double Mw, const double A[], 
					 const double G[], double Sref, int EOS_Eqn, 
					 int Cv_Eqn);
double PureEntropyP6( double V, double T, double Mw, 
					 const double AA[][8], const double rho_a[], 
					 const double G[], double Sref, double Tcrit, 
					 int Cv_Eqn);
double PureEntropyP6Q1( double V, double T, double Mw, 
					 const double AA[][7], const double G[], 
					 double Sref, double Tcrit, int Cv_Eqn );
double PureEntropyPoly( double P, double T, double Mw, 
					 const double A[], double Sref);

// ======================================================================

// =================== Pure Ideal Entropy and Enthalpy ========================
// These return pure component (NOT air) ideal enthalpy or entropy at 
// at a given temperature (and molar volume for entropy)
// ======================================================================
//		Inputs
//			V,T		Molar volume and temperature (ft3/lbmol, R).
//			Mw		Molecular weight.
//			G[]		Cv constants including beta as G[0] for C-4 or G1 as
//						G[0] if Cv is a constant for C-0. 
//			Cv_Eqn		used in the calculations, i.e. C-4 is Cv_Eqn = 4.
//		Outputs
//			Enth.	Pure component ideal enthalpy (Btu/lbmol).
//			Entr.	Pure component ideal entropy  (Btu/lbmol-R).
// ======================================================================
double IdealPEnthalpy( double T, double Mw, const double G[], int Cv_Eqn );
double IdealPEntropy( double V, double T, double Mw, const double G[], 
					int Cv_Eqn );
double IdealPEntropyx( double T, double Mw, const double G[], 
					int Cv_Eqn );

// ======================================================================


// =================== Mixed Entropy and Enthalpy =======================
// These return mixed component enthalpy or entropy at the given molar 
// volume and temperature.  They were formulated specifically for air.
// They are based on partial molar ideal contributions from pure component
// Cv's for argon, nitrogen, and oxygen, plus a residual contribution 
// calculated from an air (mixture) EOS.  See Reynolds for an explanation 
// of the integration path.
// ======================================================================
//		Inputs
//			V,T		Molar volume and temperature (ft3/lbmol, R).
//			Mw[]	Molecular weights of air, Ar, N2, and O2 respecitively.
//			A[]		EOS constants including gamma as A[0].
//			yi[]	Mole fractions of Ar (as yi[1]), N2, and O2.
//			G1[]	Cv constant, G1 = G1[0] since Cv is constant for C-0.
//			G2[],	Cv constants for N2 and O2, including beta's as the 
//			G3[]		zeroth elements of each array.
//			Href,	Reference value used to set H and S to zero at any
//			Sref		user defined reference state (T,V).
//		Outputs
//			Enth.	Air mixture enthalpy (Btu/lbmol).
//			Entr.	Air mixture entropy  (Btu/lbmol-R).
// ======================================================================
double MixedEnthalpy( double V, double T, double Mw[], 
					  const double A[], double yi[],
			 		  const double G1[], const double G2[], const double G3[],
					  double Href );
double MixedEntropy( double V, double T, double Mw[], 
					 const double A[], double yi[],
			 		 const double G1[], const double G2[], const double G3[],
					 double Sref );
// ======================================================================



// ==================== Secant Method Root Finder =======================
// Iteration algorithm to find a root using the secant method.  The
// secant method is not gauranteed to converge and thus a good initial 
// guess is mandatory.  The function Secant can be used with any 
// function that takes two arguments.  The first argument has the 
// root, and the second argument must remain constant.
// ====================================================================== 
//		Inputs
//			Target	Value of function where root (Arg1) is to be found.
//			*Arg1	Desired variable and first argument of evaluated 
//						function.  MUST have an initial guess assigned 
//						before calling Secant.
//			Arg2,	Second argument in the passed function.
//			(*Fnct)	Pointer to function F(Arg1,Arg2) that is evaluated.
//			MErr	Maximum error (stopping criteria)
//			MIter	Maximum iterations allowed (stopping criteria)
//		Outputs
//			*Arg1	Solution is in same address as initial guess.
//			Secant	Number of iterations.
// ======================================================================
int Secant( double Target, double *Arg1, double Arg2, 
		    double ( *Fnct )( double , double ), double MErr, int MIter );
// ======================================================================

// ==================== Auxiliary Functions==============================
// 
// ====================================================================== 
double Fnc_Q( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_delQdelrho( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQdT( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQdrhodT( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQ( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQdrho( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

// ====================================================================== 


// Local prototypes for non-exported functions

double HIdealC0( double T, double G1, double Mw );
double HIdealC2( double T, double G[], double Mw );
double HIdealC3( double T, double G[], double Mw );
double HIdealC4( double T, double G[], double Mw );
double HIdealC5( double T, double G[], double Mw );
double HIdealC6( double T, double G[], double Mw );
double HResidualP3( double V, double T, double A[], double Mw );
double HResidualP4( double V, double T, double A[], double Mw );
double HResidualP6( double V, double T, double Tcrit,
				    double AA[][8], double rho_a[], 
					double Mw );
double HResidualP6Q1( double V, double T, double Tcrit,
				    const double AA[][7], double Mw );
double SIdealC0( double V, double T, double G1, double Mw );
double SIdealC2( double V, double T, double G[], double Mw );
double SIdealC3( double V, double T, double G[], double Mw );
double SIdealC4( double V, double T, double G[], double Mw );
double SIdealC5( double V, double T, double G[], double Mw );
double SIdealC6( double V, double T, double G[], double Mw );
double SIdealC0x( double T, double G1, double Mw );
double SIdealC2x( double T, double G[], double Mw );
double SIdealC3x( double T, double G[], double Mw );
double SIdealC4x( double T, double G[], double Mw );
double SIdealC5x( double T, double G[], double Mw );
double SIdealC6x( double T, double G[], double Mw );
double SResidualP3( double V, double T, double A[], double Mw );
double SResidualP4( double V, double T, double A[], double Mw );
double SResidualP6( double V, double T, double Tcrit,
				    double AA[][8], double rho_a[], 
					double Mw );
double SResidualP6Q1( double V, double T, double Tcrit,
				    const double AA[][7], double Mw );



// ==================== Auxiliary Functions==============================
// 
// ====================================================================== 
double Fnc_Q( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_delQdelrho( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQdT( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQdrhodT( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQ( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_intdQdrho( double rho, double T, double Tcrit, 
			  double AA[][8], double rho_a[]);

double Fnc_Q1( double rho, double T, double Tcrit, 
			  double AA[][7]);

double Fnc_delQ1delrho( double rho, double T, double Tcrit, 
			  double AA[][7]);

double Fnc_intdQ1dT( double rho, double T, double Tcrit, 
			  double AA[][7]);

double Fnc_intdQ1drhodT( double rho, double T, double Tcrit, 
			  double AA[][7]);

double Fnc_intdQ1( double rho, double T, double Tcrit, 
			  double AA[][7]);

double Fnc_intdQ1drho( double rho, double T, double Tcrit, 
			  double AA[][7]);
// ====================================================================== 


//Cpv.h

// ============= Component Enthalpy/Entropy Functions ==================
// Enthalpy and entropy for the individual products of a hydrocarbon
// combustion reaction.  Based on polynomial Cp equation.  Vapor phase ONLY.
// Molar properties (lb-moles).
// =====================================================================
//		Inputs
//			P		Pressure (psia)
//			T		Temperature (R)
//		Outputs
//			H_		Enthalpy of the pure component (Btu/lbmole)
//			S_		Entropy of the pure component (Btu/lbmole-R)
// =====================================================================

double H_C5H12( double P, double T );
double H_CO( double P, double T );
double H_NO( double P, double T );
double H_OH( double P, double T );
double H_H( double P, double T );
double H_O( double P, double T );
double H_H2O_V( double P, double T );


double S_C5H12( double P, double T );
double S_CO( double P, double T );
double S_NO( double P, double T );
double S_OH( double P, double T );
double S_H( double P, double T );
double S_O( double P, double T );
double S_H2O_V( double P, double T );

double T_HP_C5H12( double H, double P);
double T_HP_CO( double H, double P );
double T_HP_NO( double H, double P );
double T_HP_OH( double H, double P );
double T_HP_H( double H, double P );
double T_HP_O( double H, double P );
double T_HP_H2O_V( double H, double P );

double T_SP_C5H12( double S, double P);
double T_SP_CO( double S, double P );
double T_SP_NO( double S, double P );
double T_SP_OH( double S, double P );
double T_SP_H( double S, double P );
double T_SP_O( double S, double P );
double T_SP_H2O_V( double S, double P );


// Add_Ons.h

// ==================== Mixture Molecular Weight =======================
// Molecular weight of the combustion products of a hydrocarbon mixture
// burning in air.  Complete combustion is assumed.
// =====================================================================
//		Inputs
//			HC_ratio	Ratio of hydrogen atoms to carbon atoms in 
//							hydrocarbon mixture.
//			PercentAir	Fraction of stoichiometric air supplied.  Must
//							be at least 1.00 (100%), but can have excess.
//			*CO2,*H2O,	Pointers to mole fractions of these components.
//			*Air,*Ar,		These addresses will be filled with the values
//			*N2				of the mole fractions after excecution.  
//		Outputs
//			Mw_Prod..	Molecular weight of mixture.
//			*CO2, etc.	Values of mole fractions are returned in these
//							pointer addresses after function executes.
// =====================================================================
double Mw_Products( double HC_ratio, double PercentAir, double *CO2,
				    double *H2O, double *Air, double *Ar, double *N2 );
// =====================================================================



// ================= Mixture Enthalpy and Entropy ======================
// Enthalpy and entropy of the products of complete combustion of any
// hydrocarbon in air.  Mass properties (per pound of mixture).  Call
// Mw_Products if the molar properties are needed.
// =====================================================================
//		Inputs
//			P			Pressure (psia)
//			T			Temperature (R)
//			HydroCarb	Ratio of hydrogen atoms to carbon atoms in 
//							hydrocarbon mixture.
//			PercentStoi	Fraction of stoichiometric air supplied.  Must
//							be at least 1.00 (100%) for combustion, but 
//							can have excess air.  To return the properties
//							of pure air, send a value less than 1.0.				
//		Outputs
//			H_			Enthalpy (Btu/lbm)
//			S_			Entropy (Btu/lbm-R)
// =====================================================================
double H_Products( double P, double T, double HydroCarb, double PercentStoi );
double S_Products( double P, double T, double HydroCarb, double PercentStoi );
// =====================================================================
// The following two return temperature (R) when given the enthalpy or
// entropy (in the same units as the above 2 functions) and the H to C
// ratio and percent stoichiometric air.  To get the temperature of pure
// air, send a PercentStoi less than 1.0 as above.
// =====================================================================
double TfromS_Products( double P, double S, double HydroCarb, 
						double PercentStoi );
double TfromH_Products( double P, double H, double HydroCarb, 
						double PercentStoi );