// Refrig.h
// Refrig.h is a header file.

// All information that may be "shared" between files can be placed here.  
// For example, this header file MUST contains EVERY functions.  This allows a file 
// to be complied even if a particular function is needed, but
// located in another file.   

// Perhaps an easy way to understand this is that io operations,
// math library, etc. should be included at the start of every file.
// This can be accomplished by placing these in the header file
// and including the header file in all other files in the workspace. 


#include<iostream>
#include<math.h>
#include<string.h>
#include<ctype.h>



// These are all the R134a functions that are used in the workspace

double PressureEOS_R134a(double V,  double T, const double b[], double Mw);

//FUnctions in Skeletons.cpp
int SecantPT_V_R134a( double P_Known, double *V_Target, double T_Known );
double HResidualEOS_R134a(double V, double T, const double b[], double Mw );
double HIdealEOS_R134a(double T, const double G[], double Mw);
//double PureEnthalpy_R134a(double V, double T, double MwR134a, const double R134a_b[],
//						      const double R134a_G[], double HrefR134a);

double PureEnthalpy_R134a(double V, double T, double Mw, const double R134a_b[],
						       const double R134a_G[], double Href);

int SecantHP_T_R134a( double H_Known, double *T_Target, double P_Known );
double SResidualEOS_R134a(double V, double T, const double b[], double Mw );
double SIdealEOS_R134a(double T, const double G[], double Mw);
double PureEntopy_R134a(double V, double T, double Mw, const double R134a_b[],
						                  const double R134a_G[],double Sref);

int SecantSP_T_R134a( double S_Known, double *T_Target, double P_Known );
int SecantHT_P_R134a( double H_Known, double *P_Target, double T_Known );
int SecantST_P_R134a( double S_Known, double *P_Target, double T_Known );
double SecantHsat_Tsat_L_R134a(double H_Known, double* T_Target);
double SecantSsat_Tsat_L_R134a(double S_Known, double* T_Target);

double PureSatPressure_R134a(double T);
double SecantPsat_Tsat_R134a(double P_Known, double* T_Target);
double PureSatLDensity_R134a(double T);
double PureEnthalpySat_R134a(double T);
double PureEntropySat_R134a(double T);



//R134a Functions of call.cpp
double P_R134a(double V, double T);
double V_R134a(double P, double T);
double H_R134a(double P, double T);
double S_R134a(double P, double T);
double T_HP_R134a(double H, double P);
double T_SP_R134a(double S, double P);
double P_HT_R134a(double H, double T);
double P_ST_R134a(double S, double T);
double Psat_R134a(double T);
double Tsat_R134a(double P);
double RHOsat_L_R134a(double T);
double Vsat_L_R134a(double T);
double Hsat_L_R134a(double T);
double Ssat_L_R134a(double T);
double Tsat_Hsat_L_R134a(double H);
double Tsat_Ssat_L_R134a(double S);


//R134a Functions of call.cpp
double Fnc_P_R134a(double V, double T);
double Fnc_V_R134a(double P, double T);
double Fnc_H_R134a(double P, double T);
double Fnc_S_R134a(double P, double T);
double Fnc_T_HP_R134a(double H, double P);
double Fnc_T_SP_R134a(double S, double P);
double Fnc_P_HT_R134a(double H, double T);
double Fnc_P_ST_R134a(double S, double T);
double Fnc_Psat_R134a(double T);
double Fnc_Tsat_R134a(double P);
double Fnc_RHOsat_L_R134a(double T);
double Fnc_Vsat_L_R134a(double T);
double Fnc_Hsat_L_R134a(double T);
double Fnc_Ssat_L_R134a(double T);
double Fnc_Tsat_Hsat_L_R134a(double H);
double Fnc_Tsat_Ssat_L_R134a(double S);
