
#include "Refrig.h"
// Define Gas Constants
#define RGas_kJ_kg_K 0.0815  // R in kJ/kg-K


//Define R134a Physical Property Constants

#define MwR134a		102.3		// Molecular Weight
#define TcR134a		374.23		// Critical Temperature (K) 
#define PcR134a		4060.3		// Critical Pressure (kPa)
#define VcR134a		0.00194		// m^3/kg

#define T0_R134a	273.15		//T0 Lower Limit for Cv integration

#define HrefR134a	-384.4   //
#define SrefR134a    -1.959
// Define iteration parameters

#define MaxErr 1.0E-6
#define MaxIter 100


// Define the constants for use with the EOS, Saturation Pressure, etc. 

// Define b values for use in the MBWR EOS for R134a

const double R134a_b[]={
0.000,
 3.755677E-04 ,
-8.909485E-02,
 4.408654E-05,
-2.074834E+00,
-1.016882E-03,
 2.574527E-06,
 2.142829E-02,
 1.778071E-05,
-4.016976E-08,
-2.977911E-04,
-7.481440E-08,
 1.670285E-10,
 1.255922E-06,
 4.599967
};



const double R134a_G[] ={ 
	0.00,
	3.154856E+00, 
   -1.656054E-02, 
    4.353378E-05,
   -3.754497E-08,
   -3.023189E+04
};


//
//	Here we have to write a units conversion (wrapper) program.
//	The incoming units must be converted to the EOS units.
//
//	The EOS code needs		The input and desired output is
//	T = K					T = K	
//	P = kPa					P = Mpa
//	V = m^3/kg				V = m^3/kg
//	H = kJ/kg				H = kJ/kg
//	S = kJ/kg-K				S = kJ/kg-K
//
//		
//		P = P * 1000	(Mpa -> kPa)
//		
//		  
//

double P_R134a_SI(double V, double T)
{	V = V;
	T =	T;
  	return Fnc_P_R134a( V,  T) /1000.;}
//
double V_R134a_SI(double P, double T)
{	P = P * 1000.;
	T =	T;
	return Fnc_V_R134a( P,  T) ;}
//
double H_R134a_SI(double P, double T)
{	P = P * 1000.;
	T =	T;
	return Fnc_H_R134a( P,  T) ;}
//
double S_R134a_SI(double P, double T)
{	P = P * 1000.;
	T =	T;
	return Fnc_S_R134a( P,  T) ;}
//
double T_HP_R134a_SI(double H, double P)
{	H = H;
	P = P * 1000.;
	return Fnc_T_HP_R134a( H,  P);}
//
double T_SP_R134a_SI(double S, double P)
{	S = S;
	P = P * 1000.;
	return Fnc_T_SP_R134a( S,  P);}
//
double P_HT_R134a_SI(double H, double T)
{	H = H;
	T =	T;
	return Fnc_P_HT_R134a( H,  T) /1000.;}
//
double P_ST_R134a_SI(double S, double T)
{	S = S;
	T =	T;
	return Fnc_P_ST_R134a( S,  T) /1000.;}
//
double Psat_R134a_SI(double T)
{	T =	T;
	return Fnc_Psat_R134a( T) /1000.;}
//
double Tsat_R134a_SI(double P)
{	P = P * 1000.;
	return Fnc_Tsat_R134a( P) ;}
//
double RHOsat_L_R134a_SI(double T)
{	T =	T;
	return Fnc_RHOsat_L_R134a( T);}
//
double Vsat_L_R134a_SI(double T)
{	T =	T;
	return Fnc_Vsat_L_R134a( T);}
//
double Hsat_L_R134a_SI(double T)
{	T =	T;
	return Fnc_Hsat_L_R134a( T);}
//
double Ssat_L_R134a_SI(double T)
{	T =	T;
	return Fnc_Ssat_L_R134a( T);}
//
double Tsat_Hsat_L_R134a_SI(double H)
{	H = H;
	return Fnc_Tsat_Hsat_L_R134a( H);}
//
double Tsat_Ssat_L_R134a_SI(double S)
{	S = S;
	return Fnc_Tsat_Ssat_L_R134a( S);}
//

// P_R134a.  Function call to determine the Pressure of R134a when V and T are known.
// Here a call is made to PressureEOS_R134a which is the EOS for R134a.
// This function is located in Refrig_Skeletons and it returns a value for Pressure
// which will be assigned to P_R134a. 

double Fnc_P_R134a(double V, double T)
{  	return PressureEOS_R134a (V, T, R134a_b, MwR134a);}


// V_R134a.  Function call to determine Volume of R134a when P and T are known.
// here a call is made to V_EOS (in Refrig_Calls) which will use 
// the Secant Method (Secant_R143a in Refrig_Skeletons) to determine the Volume.
// The ideal gas law supplies the initial Vguess
// V_EOS also supplies the known P and known T to the Secant Method.
// The Secant Method will call PressureEOS_R134a with estimated V values.
// Solution is reached when Vguess gives P (from the EOS) = P known.
//
//
// ******
// At this point the user may be asking why not just move V_EOS to 
// Refrig_Skeletons and imbed the Secant Method in V_EOS ?
//
// This could be done,  
// But it truns out that would require   
// needles duplication.  
//
// Eventually as shown in  
// in Reynolds_Skeletons we will have just one V_EOS function and in 
// Reynolds_Calls we will have just one Secant function.
// Pointers are used for the proper EOS  
// 

double Fnc_V_R134a(double P, double T)
{
	
	// Curve Fit To Guess V From (T,P)
	double A= -1.3588E-05;
	double B= 0.009669021;	
	double C= 0.000218785;	
	double D= 0.011949443;	
	double E= 0.077417516;	
	double F= -1.681089185;	
 	double V_Guess = A*T*T +B*T +C/(P*P) +D/P +E*T/P +F;
	if (V_Guess < 0.01) V_Guess = 0.01;
    if (V_Guess > 2.0) V_Guess = 2.0; 
	
;
	int iterations;
	
	iterations = SecantPT_V_R134a( P, &V_Guess, T);	
	if (iterations < MaxIter) return V_Guess;
	 else return -iterations;
    
}



double Fnc_H_R134a(double P, double T)
{
	return PureEnthalpy_R134a(Fnc_V_R134a(P,T), T, MwR134a,R134a_b,R134a_G,
													         HrefR134a);
}

double Fnc_S_R134a(double P, double T)
{
	return PureEntopy_R134a(Fnc_V_R134a(P,T), T, MwR134a,R134a_b,R134a_G,
													         SrefR134a);
}


double Fnc_T_HP_R134a(double H, double P)
{
	
	// Curve Fit To Guess T From (P,H)

	double A=-7.11E-08;
	double B=0.047598858; 	
	double C=-0.000591977; 
	double D=1.633949245; 
	double E=-7.44166E-05; 
	double F=-289.85;
 	double T_Guess = A*P*P +B*P +C*H*H +D*H +E*P*H +F;
	if (T_Guess < -30 + 273.15) T_Guess = -30 + 273.15;
    if (T_Guess > 230 + 273.15) T_Guess = 230 + 273.15; 
	
	//cout<<"T_Guess "<<T_Guess<<"\n";

	int iterations = SecantHP_T_R134a(H, &T_Guess, P);	
	if (iterations < MaxIter) return T_Guess;
	 else return -iterations;
  
}


double Fnc_T_SP_R134a(double S, double P)
{                 
	
// Curve Fit To Guess T From (P,S)
	double A= -2.46631E-05;
	double B= 0.126082176;	
	double C= -654.4780235;	
	double D= -640.94746;	
	double E= -0.017679433;	
	double F= 811.6811474;	
 	double T_Guess = A*P*P +B*P +C/(S*S) +D/S +E*P/S +F;
	if (T_Guess < -30 + 273.15) T_Guess = -30 + 273.15;
    if (T_Guess > 230 + 273.15) T_Guess = 230 + 273.15; 
		
//	cout<<"T_Guess "<<T_Guess -273.15<<"\n";

	int iterations = SecantSP_T_R134a(S, &T_Guess, P);	
	if (iterations < MaxIter) return T_Guess;
	 else return -iterations;
}

double Fnc_P_HT_R134a(double H, double T)
{

	
	
	// Curve Fit To Guess P From (T,H)

	double A= 0.02487099;
	double B= -145.1492776;	
	double C= -0.403836722;	
	double D= 133.4972449;	
	double E= 0.452041643;	
	double F= -7.113450886;	
 
	
	double P_Guess = A*T*T +B*T +C*H*H +D*H +E*T*H +F;
	if (P_Guess < 30 ) P_Guess = 30 ;
    if (P_Guess > 3800) P_Guess = 3800; 
	
	
//	cout<<"P_Guess= "<<P_Guess<<"\n";
	int iterations = SecantHT_P_R134a(H, &P_Guess, T);	
	if (iterations < MaxIter) return P_Guess;
	 else return -iterations;
  
}


double Fnc_P_ST_R134a(double S, double T)
{
	
	
	// Curve Fit To Guess P From (T,S)

	double A= 0.144571617;
	double B= 82.39774753;	
	double C= 9435.109253;	
	double D= -10867.32883;	
	double E= -87.42598333;	
	double F= -1209.532189;	

	double P_Guess = A*T*T +B*T +C*S*S +D*S +E*T*S +F;
	if (P_Guess < 30 ) P_Guess = 30 ;
    if (P_Guess > 3800) P_Guess = 3800; 
	

 //	cout<<"P_Guess= "<<P_Guess<<"\n";
	int iterations = SecantST_P_R134a(S, &P_Guess, T);	
	if (iterations < MaxIter) return P_Guess;
	 else return -iterations;
  
}

double Fnc_Psat_R134a(double T)
{
	return PureSatPressure_R134a(T);
}

double Fnc_Tsat_R134a(double P)
{

	
	// Curve Fit To Guess Tsat From Psat

	
	double A= -1.60E-05;
	double B= 0.092786262;	
	double C= 229.1357128;
	double T_Guess = A*P*P +B*P +C;
	 
	int iterations;
	
//	cout<<"T_Guess :"<<T_Guess<<"\n";
	 iterations = (int)SecantPsat_Tsat_R134a(P, &T_Guess);	
	if (iterations < MaxIter) return T_Guess;
	 else return -iterations;
}

double Fnc_RHOsat_L_R134a(double T)
{
	return PureSatLDensity_R134a(T);
}


double Fnc_Vsat_L_R134a(double T)
{
	return 1.0/Fnc_RHOsat_L_R134a(T);
}


double Fnc_Hsat_L_R134a(double T)
{
  	return PureEnthalpySat_R134a(T);
}



double Fnc_Ssat_L_R134a(double T)
{
 	return PureEntropySat_R134a(T);
}


double Fnc_Tsat_Hsat_L_R134a(double H)
{

	// Guess Tsat from Hsat known 

	double T_Guess = H/(RGas_kJ_kg_K *24.29115431) + 173.15;
	 
	int iterations;
	
//	cout<<"T_Guess :"<<T_Guess-273.15<<"\n";
	 iterations = (int)SecantHsat_Tsat_L_R134a(H, &T_Guess);	
	if (iterations < MaxIter) return T_Guess;
	 else return -iterations;
}



double Fnc_Tsat_Ssat_L_R134a(double S)
{

	// Guess Tsat from Ssat known 

	double T_Guess = exp(S/(RGas_kJ_kg_K*3.6)) + 173.15;
	 
	int iterations;
	
//	cout<<"T_Guess :"<<T_Guess-273.15<<"\n";
						
	 iterations = (int)SecantSsat_Tsat_L_R134a(S, &T_Guess);	
	if (iterations < MaxIter) return T_Guess;
	 else return -iterations;
}

